from typing import Union

from pyjlyric.model import LyricPage, WithUrlText


class EvestaLyricPage(LyricPage):
    artist: Union[str, WithUrlText]
    composer: str
    lyricist: str
    arranger: None



from typing import Union

from pyjlyric.model import LyricPage, WithUrlText


class UtanetLyricPage(LyricPage):
    artist: Union[str, WithUrlText]
    composer: Union[str, WithUrlText]
    lyricist: Union[str, WithUrlText]
    arranger: Union[None, str, WithUrlText] = None



from collections.abc import Iterator
from typing import List, Type

from pydantic import RootModel, field_validator
from typing_extensions import Self

from pyjlyric.model import LyricPage, WithUrlText


class HoickLyricPage(LyricPage):
    artist: None
    composer: List[WithUrlText]
    lyricist: List[WithUrlText]
    arranger: None


class HoickLyricData(RootModel[List[str]]):
    def __iter__(self: Self) -> Iterator[str]:  # type: ignore[override]
        return iter(self.root)

    @field_validator("root")
    @classmethod
    def validate(cls: Type[Self], v: List[str]) -> List[str]:
        return [i for i in v if isinstance(i, str)]



"""Pydantic models for lyric data."""
from typing import List, Union

from pydantic import BaseModel, HttpUrl


class WithUrlText(BaseModel):
    """WIP."""

    link: HttpUrl
    text: str


_LyricSection = List[str]


class LyricPage(BaseModel):
    """WIP."""

    title: str
    page_url: HttpUrl
    pageid: str

    artist: Union[str, WithUrlText, List[str], List[WithUrlText], None] = None
    composer: Union[str, WithUrlText, List[str], List[WithUrlText], None] = None
    lyricist: Union[str, WithUrlText, List[str], List[WithUrlText], None] = None
    arranger: Union[str, WithUrlText, List[str], List[WithUrlText], None] = None

    lyric_sections: List[_LyricSection]
